@@ -37,6 +37,23 @@ class DalyBmsComponent : public PollingComponent, public uart::UARTDevice {
   void set_cells_number_sensor(sensor::Sensor *cells_number) { cells_number_ = cells_number; }
   void set_temperature_1_sensor(sensor::Sensor *temperature_1_sensor) { temperature_1_sensor_ = temperature_1_sensor; }
   void set_temperature_2_sensor(sensor::Sensor *temperature_2_sensor) { temperature_2_sensor_ = temperature_2_sensor; }
+  void set_cell_1_voltage_sensor(sensor::Sensor *cell_1_voltage) { cell_1_voltage_ = cell_1_voltage; }
+  void set_cell_2_voltage_sensor(sensor::Sensor *cell_2_voltage) { cell_2_voltage_ = cell_2_voltage; }
+  void set_cell_3_voltage_sensor(sensor::Sensor *cell_3_voltage) { cell_3_voltage_ = cell_3_voltage; }
+  void set_cell_4_voltage_sensor(sensor::Sensor *cell_4_voltage) { cell_4_voltage_ = cell_4_voltage; }
+  void set_cell_5_voltage_sensor(sensor::Sensor *cell_5_voltage) { cell_5_voltage_ = cell_5_voltage; }
+  void set_cell_6_voltage_sensor(sensor::Sensor *cell_6_voltage) { cell_6_voltage_ = cell_6_voltage; }
+  void set_cell_7_voltage_sensor(sensor::Sensor *cell_7_voltage) { cell_7_voltage_ = cell_7_voltage; }
+  void set_cell_8_voltage_sensor(sensor::Sensor *cell_8_voltage) { cell_8_voltage_ = cell_8_voltage; }
+  void set_cell_9_voltage_sensor(sensor::Sensor *cell_9_voltage) { cell_9_voltage_ = cell_9_voltage; }
+  void set_cell_10_voltage_sensor(sensor::Sensor *cell_10_voltage) { cell_10_voltage_ = cell_10_voltage; }
+  void set_cell_11_voltage_sensor(sensor::Sensor *cell_11_voltage) { cell_11_voltage_ = cell_11_voltage; }
+  void set_cell_12_voltage_sensor(sensor::Sensor *cell_12_voltage) { cell_12_voltage_ = cell_12_voltage; }
+  void set_cell_13_voltage_sensor(sensor::Sensor *cell_13_voltage) { cell_13_voltage_ = cell_13_voltage; }
+  void set_cell_14_voltage_sensor(sensor::Sensor *cell_14_voltage) { cell_14_voltage_ = cell_14_voltage; }
+  void set_cell_15_voltage_sensor(sensor::Sensor *cell_15_voltage) { cell_15_voltage_ = cell_15_voltage; }
+  void set_cell_16_voltage_sensor(sensor::Sensor *cell_16_voltage) { cell_16_voltage_ = cell_16_voltage; }
+
   // TEXT_SENSORS
   void set_status_text_sensor(text_sensor::TextSensor *status_text_sensor) { status_text_sensor_ = status_text_sensor; }
   // BINARY_SENSORS
@@ -52,11 +69,14 @@ class DalyBmsComponent : public PollingComponent, public uart::UARTDevice {
   void update() override;
 
   float get_setup_priority() const override;
+  void set_address(uint8_t address) { this->addr_ = address; }
 
  protected:
   void request_data_(uint8_t data_id);
   void decode_data_(std::vector<uint8_t> data);
 
+  uint8_t addr_;
+
   sensor::Sensor *voltage_sensor_{nullptr};
   sensor::Sensor *current_sensor_{nullptr};
   sensor::Sensor *battery_level_sensor_{nullptr};
@@ -72,6 +92,22 @@ class DalyBmsComponent : public PollingComponent, public uart::UARTDevice {
   sensor::Sensor *cells_number_{nullptr};
   sensor::Sensor *temperature_1_sensor_{nullptr};
   sensor::Sensor *temperature_2_sensor_{nullptr};
+  sensor::Sensor *cell_1_voltage_{nullptr};
+  sensor::Sensor *cell_2_voltage_{nullptr};
+  sensor::Sensor *cell_3_voltage_{nullptr};
+  sensor::Sensor *cell_4_voltage_{nullptr};
+  sensor::Sensor *cell_5_voltage_{nullptr};
+  sensor::Sensor *cell_6_voltage_{nullptr};
+  sensor::Sensor *cell_7_voltage_{nullptr};
+  sensor::Sensor *cell_8_voltage_{nullptr};
+  sensor::Sensor *cell_9_voltage_{nullptr};
+  sensor::Sensor *cell_10_voltage_{nullptr};
+  sensor::Sensor *cell_11_voltage_{nullptr};
+  sensor::Sensor *cell_12_voltage_{nullptr};
+  sensor::Sensor *cell_13_voltage_{nullptr};
+  sensor::Sensor *cell_14_voltage_{nullptr};
+  sensor::Sensor *cell_15_voltage_{nullptr};
+  sensor::Sensor *cell_16_voltage_{nullptr};
 
   text_sensor::TextSensor *status_text_sensor_{nullptr};
 
