@@ -2,10 +2,8 @@ import esphome.codegen as cg
 import esphome.config_validation as cv
 from esphome.components import sensor, spi
 from esphome.const import (
-    CONF_ID,
     CONF_MAINS_FILTER,
     DEVICE_CLASS_TEMPERATURE,
-    ICON_EMPTY,
     STATE_CLASS_MEASUREMENT,
     UNIT_CELSIUS,
 )
@@ -23,15 +21,14 @@ FILTER = {
 
 CONFIG_SCHEMA = (
     sensor.sensor_schema(
-        UNIT_CELSIUS,
-        ICON_EMPTY,
-        1,
-        DEVICE_CLASS_TEMPERATURE,
-        STATE_CLASS_MEASUREMENT,
+        MAX31856Sensor,
+        unit_of_measurement=UNIT_CELSIUS,
+        accuracy_decimals=1,
+        device_class=DEVICE_CLASS_TEMPERATURE,
+        state_class=STATE_CLASS_MEASUREMENT,
     )
     .extend(
         {
-            cv.GenerateID(): cv.declare_id(MAX31856Sensor),
             cv.Optional(CONF_MAINS_FILTER, default="60HZ"): cv.enum(
                 FILTER, upper=True, space=""
             ),
@@ -43,8 +40,7 @@ CONFIG_SCHEMA = (
 
 
 async def to_code(config):
-    var = cg.new_Pvariable(config[CONF_ID])
+    var = await sensor.new_sensor(config)
     await cg.register_component(var, config)
     await spi.register_spi_device(var, config)
-    await sensor.register_sensor(var, config)
     cg.add(var.set_filter(config[CONF_MAINS_FILTER]))
