@@ -105,7 +105,11 @@ class SPIComponent : public Component {
   void write_byte(uint8_t data) {
 #ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
+#ifdef USE_RP2040
+      this->hw_spi_->transfer(data);
+#else
       this->hw_spi_->write(data);
+#endif
       return;
     }
 #endif  // USE_SPI_ARDUINO_BACKEND
@@ -116,7 +120,11 @@ class SPIComponent : public Component {
   void write_byte16(const uint16_t data) {
 #ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
+#ifdef USE_RP2040
+      this->hw_spi_->transfer16(data);
+#else
       this->hw_spi_->write16(data);
+#endif
       return;
     }
 #endif  // USE_SPI_ARDUINO_BACKEND
@@ -130,7 +138,11 @@ class SPIComponent : public Component {
 #ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       for (size_t i = 0; i < length; i++) {
+#ifdef USE_RP2040
+        this->hw_spi_->transfer16(data[i]);
+#else
         this->hw_spi_->write16(data[i]);
+#endif
       }
       return;
     }
@@ -145,7 +157,11 @@ class SPIComponent : public Component {
 #ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       auto *data_c = const_cast<uint8_t *>(data);
+#ifdef USE_RP2040
+      this->hw_spi_->transfer(data_c, length);
+#else
       this->hw_spi_->writeBytes(data_c, length);
+#endif
       return;
     }
 #endif  // USE_SPI_ARDUINO_BACKEND
@@ -178,7 +194,11 @@ class SPIComponent : public Component {
       if (this->miso_ != nullptr) {
         this->hw_spi_->transfer(data, length);
       } else {
+#ifdef USE_RP2040
+        this->hw_spi_->transfer(data, length);
+#else
         this->hw_spi_->writeBytes(data, length);
+#endif
       }
       return;
     }
@@ -205,7 +225,11 @@ class SPIComponent : public Component {
       } else if (CLOCK_POLARITY && CLOCK_PHASE) {
         data_mode = SPI_MODE3;
       }
+#ifdef USE_RP2040
+      SPISettings settings(DATA_RATE, static_cast<BitOrder>(BIT_ORDER), data_mode);
+#else
       SPISettings settings(DATA_RATE, BIT_ORDER, data_mode);
+#endif
       this->hw_spi_->beginTransaction(settings);
     } else {
 #endif  // USE_SPI_ARDUINO_BACKEND
